/* File: create_sphere_file.c */
   
/*
 * Richard Duncan
 * Institute for Signal and Information Processing
 * December 1995
 *
 * This procedure converts a raw speech data file
 * into the NIST sphere data file format
 *
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef __SP_SPHERE_H_
#define __SP_SPHERE_H_
#include <sp/sphere.h>
#endif

#ifndef __MAIN_H_
#define __MAIN_H_
#include "main.h"
#endif

#ifndef __CREATE_SPHERE_FILE_H_
#define __CREATE_SPHERE_FILE_H_
#include "create_sphere_file.h"
#endif

#ifndef __READ_AND_DEBIAS_DATA_H_
#define __READ_AND_DEBIAS_DATA_H_
#include "read_and_debias_data.h"
#endif

#ifndef __READ_SESSION_INFO_H_
#define __READ_SESSION_INFO_H_
#include "read_session_info.h"
#endif

#ifndef __READ_VAL_FILE_H_
#define __READ_VAL_FILE_H_
#include "read_val_file.h"
#endif


int create_sphere_file(char *raw_data_filename,
		       char *sphere_filename,
		       char *val_filename,
		       FILE *session_info_fp
		       ) {
  
  struct header_t *header;
  register FILE *sphere_fp;
  long sample_count, sample_n_bytes, sample_sig_bits, channel_count;
  long begin, end;
  int temp_error;
  int channel_number;
  char *temp;
  
  char field_name[32];
  char field_value[32];

  char *data;

  long count,size;
  
  /*
   * create the header
   *
   */
  header = sp_create_header();
  if (header == HDRNULL) {
    fprintf(stderr,"Error: Could not create header, exiting!\n");
    exit(21);
  }

  /*
   * read and debias the data
   *
   */

  data = read_and_debias_data(header, raw_data_filename, &count, &size);

  printf("\ttransfering %d samples of data\n",count);
  
  /*
   * set the crucial values that are constant
   *
   */
  sample_n_bytes = sizeof(INT_2);
  channel_count = 1;
  sample_sig_bits = 8 * sample_n_bytes;
  
  /*
   * set the crucial fields for a sphere file:
   *     sample_n_bytes: length of each sample (16 bit = 2 bytes)
   *     channel_count: number of channels in file (1)
   *
   */
  
  temp_error = sp_add_field(header, "sample_n_bytes",
			    T_INTEGER, (char *) &sample_n_bytes);
  if (temp_error >= 0)
    temp_error = sp_add_field(header, "sample_sig_bits",
			      T_INTEGER, (char *) &sample_sig_bits);
  if (temp_error >= 0)
    temp_error = sp_add_field(header, "channel_count",
			      T_INTEGER, (char *) &channel_count);
  if (temp_error < 0) {
    fprintf(stderr,"Error adding crucial fields to header, exiting!\n");
    exit(22);
  }

    
  /*
   * read in the session info
   *
   */

  read_session_info(header, session_info_fp, val_filename);

  /*
   * read in the val file
   *
   */

  read_val_file(header, val_filename);

  /*
   * determine the speaker's sex from the raw_data_filename
   *
   */

  if (strchr(raw_data_filename,(int)'/')){
    temp = strrchr(raw_data_filename,(int)'/') + 1;
  }
  else {
    temp = raw_data_filename;
  }

  switch (*temp){
  case 'f':
    strcpy(field_value, "female");
    break;
  case 'm':
    strcpy(field_value, "male");
    break;
  default:
    fprintf(stderr, field_read_error, "speaker_sex");
    exit(53);
  }
  if(sp_add_field(header, "speaker_sex", T_STRING, field_value) < 0){
    fprintf(stderr, field_error, "speaker_sex");
    exit(54);
  }
  
  /*
   * determine the session_utterance_number from the raw filename
   *
   */
  
  strcpy(field_value,strstr(raw_data_filename, "_")+1);
  *(strstr(field_value, "_")) = (char) NULL;
  if(sp_add_field(header, "session_utterance_number",
		  T_STRING, field_value) < 0){
    fprintf(stderr, field_error, "session_utterance_number");
    exit(54);
  }
  /*
   * calculate the microphone from the raw_data_filename
   *
   */
  {
    /* changing the algorithm to parse to the period, then search
       backwards for the _c */

    int channel_number;

    temp = strstr(raw_data_filename,".");
    while ((temp > raw_data_filename) &&
	   (*temp != '_') &&
	   (*(temp+1) != 'c')){
      temp--;
    }

    if(temp < raw_data_filename){
      fprintf(stderr,field_error,"microphone");
    }
    channel_number = atoi(temp+2);
    
    switch (channel_number) {
    case 0:
      strcpy(field_value, microphone_0);
      break;
    case 1:
      strcpy(field_value, microphone_1);
      break;
    default:
      fprintf(stderr, field_read_error, "microphone");
      exit(53);
    }
  
    if(sp_add_field(header, "microphone", T_STRING, field_value) < 0){
      fprintf(stderr, field_error, "microphone");
      exit(54);
    }
  }
  /*
   * Add the two constant values, sample_byte_format & sample_byte_coding
   *
   */
  strcpy(field_value, "10");
  if(sp_add_field(header, "sample_byte_format", T_STRING, field_value) < 0){
    fprintf(stderr, field_error, "sample_byte_format");
    exit(54);
  }
  strcpy(field_value, "pcm");
  if(sp_add_field(header, "sample_coding", T_STRING, field_value) < 0){
    fprintf(stderr, field_error, "sample_coding");
    exit(54);
  }
  
  /*
   * open the empty sphere file
   *
   */
  sphere_fp = fopen(sphere_filename, "w");

  /*
   * write the header to the empty sphere file
   *
   */
  {
    long databytes, hbytes;
    
    rewind(sphere_fp);
    if ((sp_write_header(sphere_fp,header,&hbytes,&databytes)) < 0) {
      fprintf(stderr,"Error: sp_write_header failed, exiting!\n");
      exit(24);
    }
  }
  /*
   * write the data
   *
   */
  if(fwrite((void *)data, size, count, sphere_fp)!=
     count){
    fprintf(stderr, file_write_error, sphere_filename);
  }
  
  
  /*
   * close the sphere file
   *
   */

  fclose(sphere_fp);

  /*
   * free up the header space
   *
   */

  sp_close_header(header);

  /*
   * free up the memory from the data
   *
   */

  free(data);


  /*
   * Test the sphere file by opening it
   *
   */
   {
   SP_FILE *sfp;
   
   sfp = sp_open(sphere_filename, "r");
   if (sfp == (SP_FILE *)NULL){
   fprintf(stderr, "Bad sphere file, exiting!\n");
   exit(2);
   }
   sp_close(sfp);
  }
  
}
